#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

UNKNOWN = {
    'uuid': 'unknown-uuid',
    'name': 'unknown-name',
    'resource_class': 'unknown-class',
    'config': {},
    'owner_project_id': 'unknown-owner',
    'lease_uuid': 'unknown-lease',
    'lessee_project_id': 'unknown-lessee',
    'provision_state': 'unknown-provision-state',
    'power_state': 'unknown-power-state'
}
